﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Emgu.CV;
using Emgu.CV.Structure;
using Emgu.CV.CvEnum;
using System.Runtime.InteropServices;
using System.Drawing;

namespace GestaltSegmentation
{
    class Gestalt
    {
        static Image<Bgr, Byte> image;
        static Image<Bgr, Byte> img;
        static Image<Bgr, Byte> imgproxit;
        static Point prev;
        static int step = 0;
        static List<Color> colorList = new List<Color>() { Color.Plum,Color.PowderBlue,Color.RosyBrown,Color.Salmon,Color.Snow,Color.SteelBlue};
        static List<Point> newlyAddedPoint=new List<Point>();
        static List<Point> allPoint = new List<Point>();
        static int colorIndex = 0;
        static Tuple<double,double> direction;    //unit vector along motion        
        static double nEnergy = 0;
        static List<Tuple<double, int, double>> constantLawContinuumEnergy = new List<Tuple<double, int, double>>();
        static List<Tuple<double, int, double>> continuumLawContinuumEnergy = new List<Tuple<double, int, double>>();
        static List<Tuple<double, int, double>> constantLawConstantEnergy = new List<Tuple<double, int, double>>();
        static List<Tuple<double, int, double>> continuumLawConstantEnergy = new List<Tuple<double, int, double>>();
        static bool[,] sample;
        static List<Point> cluster;
        const string problem = "circleTriangle";
        static int problemNo;
        static int maxObject1 = 0, maxObject2 = 0, maxObject3=0;

        static void checkContrastAndMark(Point p)
        {
            /*  find neighbors of P and compare the color
             *  if there is a mark changes it any one of them
             *  then we put it as red color else we continue along
             *  its neighbor. If it is marked as red then we move
             *  along */
            List<Point> neighbhors = giveNeighbhors(p);
            foreach (Point t in neighbhors)
            {
                //check for contrast with p
                int b = Math.Abs(image.Data[p.Y, p.X, 0] - image.Data[t.Y, t.X, 0]);
                int g = Math.Abs(image.Data[p.Y, p.X, 1] - image.Data[t.Y, t.X, 1]);
                int r = Math.Abs(image.Data[p.Y, p.X, 2] - image.Data[t.Y, t.X, 2]);
                if ((r + g + b) / 3 > Constant.contrastThreshold)
                {
                    //point is a border. marking as red
                    img.Data[p.Y, p.X, 0] = 0;
                    img.Data[p.Y, p.X, 1] = 0;
                    img.Data[p.Y, p.X, 2] = 255;
                    break;
                }
            }
        }

        static void segment()
        {
            for (int x = 0; x < image.Width; x++)
            {
                for (int y = 0; y < image.Height; y++)
                {
                    checkContrastAndMark(new Point(x,y));
                }
            }
        }

        static bool isMarked(Point p)
        {
            if (img.Data[p.Y, p.X, 0] == 0 && img.Data[p.Y, p.X, 1] == 0 && img.Data[p.Y, p.X, 2] == 255)
                return true;
            return false;
        }
        static bool isMarkedP(Point p)
        {
            if (imgproxit.Data[p.Y, p.X, 0] == 255 && imgproxit.Data[p.Y, p.X, 1] == 255 && imgproxit.Data[p.Y, p.X, 2] == 255)
                return false;
            return true;
        }
        static bool isMarkedXY(int X, int Y)
        {
            if (img.Data[Y, X, 0] == 255 && img.Data[Y, X, 1] == 255 && img.Data[Y, X, 2] == 255)
                return false;
            return true;
        }

        static void travel(Point p)
        {            
            /*stores the cluster around p*/            
            Stack<Point> stack = new Stack<Point>();
            cluster.Add(p);
            stack.Push(p);
            while (stack.Count!=0)
            {
                Point t=stack.Pop();
                List<Point> ngbr = giveNeighbhors(t);
                foreach (Point u in ngbr)
                {                    
                    if (sample[u.Y,u.X] && !isMarked(u) ) //not red and not used
                    {
                        sample[u.Y, u.X] = false;   //use it
                        cluster.Add(u);
                        stack.Push(u);
                    }
                }
            }
        }


        static void GestaltConstantColor()
        {
            /* assumes rgb image is in image. This function picks up a 
             * random point and then start exploring region around it
             * and storing points till it reaches a boundary.*/
            sample = new bool[image.Height, image.Width];
            cluster = new List<Point>();
            List<List<Point>> clusters = new List<List<Point>>();            
            Point p;

            for (int y = 0; y < image.Height; y++)
            {
                for (int x = 0; x < image.Width; x++)
                    sample[y, x] = true;
            }

            y_gsaved = 0;
            while ((p=givePoints(sample)).X!=-1)
            {                
                sample[p.Y,p.X]=false;
                if (isMarked(p))
                    continue;
                travel(p);
                if (cluster.Count > Constant.supportThreshold)
                {                    
                    saveSegments(cluster);
                    maxObject1++;
                    if (maxObject1 >= Constant.maxObjects)
                        return;
                }
                cluster.Clear();
                clusters.Add(cluster);                
            }
            Console.WriteLine("Saving Clusters");            
        }

        static void travelProximity(Point p)
        {
            /*stores the cluster around p*/
            Stack<Point> stack = new Stack<Point>();
            cluster.Add(p);
            stack.Push(p);
            while (stack.Count != 0)
            {
                Point t = stack.Pop();
                List<Point> ngbr = giveNeighbhors(t);
                foreach (Point u in ngbr)
                {
                    if (isMarkedP(u) && sample[u.Y, u.X]) //not used
                    {
                        sample[u.Y, u.X] = false;   //use it
                        cluster.Add(u);
                        stack.Push(u);
                    }
                }
            }
        }
        

        static void GestaltProximity()
        {
            sample = new bool[imgproxit.Height, imgproxit.Width];
            cluster = new List<Point>();
            List<List<Point>> clusters = new List<List<Point>>();
            Point p;

            for (int y = 0; y < imgproxit.Height; y++)
            {
                for (int x = 0; x < imgproxit.Width; x++)
                    sample[y, x] = true;
            }

            y_saved = 0;
            while ((p = giveMarkedPoints(sample)).X != -1)
            {
                sample[p.Y, p.X] = false;
                travelProximity(p);
                if (cluster.Count > Constant.supportThreshold)
                {
                    saveSegmentsProximity(cluster);
                    maxObject3++;
                    if (maxObject3 >= Constant.maxObjects)
                        return;
                }
                cluster.Clear();
                clusters.Add(cluster);
            }
            Console.WriteLine("Saving Proximity Clusters");
        }
        static int y_gsaved = 0;
        static Point givePoints(bool[,] sample)
        {
            for (int y = y_gsaved; y < image.Height; y++)
            {
                for (int x = 0; x < image.Width; x++)
                {
                    if (sample[y, x])
                    {
                        y_gsaved = y;
                        return new Point(x, y);
                    }
                }
            }
            return new Point(-1, -1); //dummy point
        }
        static int y_saved = 0;
        static Point giveMarkedPoints(bool[,] sample)
        {
            for (int y = y_saved; y < image.Height; y++)
            {
                for (int x = 0; x < image.Width; x++)
                {
                    if (isMarkedXY(x, y) && sample[y, x])
                    {
                        y_saved = y;
                        return new Point(x, y);
                    }
                }
            }
            return new Point(-1, -1); //dummy point
        }

        static int index = 0;
        static void saveSegments(List<Point> clt)
        {            
            //foreach (List<Point> clt in clusters)
            //{
                Image<Bgr, byte> itmp = new Image<Bgr, byte>(image.Width,image.Height);
                foreach (Point pt in clt)
                {
                    itmp.Data[pt.Y, pt.X, 0] = image.Data[pt.Y, pt.X, 0];
                    itmp.Data[pt.Y, pt.X, 1] = image.Data[pt.Y, pt.X, 1];
                    itmp.Data[pt.Y, pt.X, 2] = image.Data[pt.Y, pt.X, 2];
                }
                    itmp.Save(@"C:\dataset\Ans\" + problemNo.ToString() + @"\ColorConstancy\" + index.ToString() + ".png");                
                index++;
                constantLawConstantEnergy.Add(constantComplexity(itmp));
                constantLawContinuumEnergy.Add(continuityComplexity(itmp.Convert<Gray, Byte>()));
            //}
        }

        static int index1 = 0;
        static void saveSegmentsProximity(List<Point> clt)
        {
            //foreach (List<Point> clt in clusters)
            //{
            Image<Bgr, byte> itmp = new Image<Bgr, byte>(image.Width, image.Height);
            foreach (Point pt in clt)
            {
                itmp.Data[pt.Y, pt.X, 0] = imgproxit.Data[pt.Y, pt.X, 0];
                itmp.Data[pt.Y, pt.X, 1] = imgproxit.Data[pt.Y, pt.X, 1];
                itmp.Data[pt.Y, pt.X, 2] = imgproxit.Data[pt.Y, pt.X, 2];
            }
            //CvInvoke.cvShowImage("tempimage", image);
            //CvInvoke.cvWaitKey(0);
            CvInvoke.cvShowImage("tempimage", itmp);
            CvInvoke.cvWaitKey(0);
            itmp.Save(@"C:\dataset\Ans\" + problemNo.ToString() + @"\Proximity\" + index1.ToString() + ".png");
            if (index1 != 0)
            {
                MemStorage n = new MemStorage();
                Image<Gray, Byte> tmpimage = itmp.Convert<Gray, Byte>();
                Contour<Point> contour = new Contour<Point>(n);
                contour = tmpimage.FindContours();
                //contour.BoundingRectangle      
            }
            //if (problemNo >= 51 && problemNo <=55)
            //{

            //}
            index1++;
            //constantLawConstantEnergy.Add(constantComplexity(itmp));
            //constantLawContinuumEnergy.Add(continuityComplexity(itmp.Convert<Gray, Byte>()));
            //}
        }
        static List<Point> giveNeighbhors(Point p)
        {
            List<Point> neighbhors = new List<Point>();
            if (p.X != 0)
            {
                //take left point and add
                neighbhors.Add(new Point(p.X - 1, p.Y));
            }

            if (p.Y != 0)
            {
                //take top point and add
                neighbhors.Add(new Point(p.X, p.Y - 1));
            }

            if (p.X != image.Width - 1)
            {
                //take right point and add
                neighbhors.Add(new Point(p.X + 1, p.Y));
            }

            if (p.Y != image.Height - 1)
            {
                //take bottom point and add
                neighbhors.Add(new Point(p.X, p.Y + 1));
            }

            if (p.X != 0 && p.Y != 0)
            {
                //take top-left
                neighbhors.Add(new Point(p.X - 1, p.Y - 1));
            }

            if (p.X != 0 && p.Y != image.Height - 1)
            {
                //take bottom-left
                neighbhors.Add(new Point(p.X - 1, p.Y + 1));
            }

            if (p.X != image.Width - 1 && p.Y != 0)
            {
                //take top-right
                neighbhors.Add(new Point(p.X + 1, p.Y - 1));
            }

            if (p.X != image.Width - 1 && p.Y != image.Height - 1)
            {
                //take bottom-right
                neighbhors.Add(new Point(p.X + 1, p.Y + 1));
            }
            return neighbhors;
        }

        static Point moveKStep(Point p, int pos, List<Point> lp)
        {
            lp.Add(p);
            if (pos == 0)
                return p;
            pos--;
            List<Point> neighbhors = giveNeighbhors(p);

            List<Tuple<double, double>> options = new List<Tuple<double, double>>();
            foreach (Point t in neighbhors)
            {
                int b = img.Data[t.Y, t.X, 0];
                int g = img.Data[t.Y, t.X, 1];
                int r = img.Data[t.Y, t.X, 2];
                if (b == 0 && g == 0 && r == 255)   //red pixel
                {
                    if (t.X != prev.X || t.Y != prev.Y) //not previous location
                    {
                        //add unit vector along it
                        double norm = Math.Sqrt((t.X - p.X) * (t.X - p.X) + (t.Y - p.Y) * (t.Y - p.Y));
                        options.Add(new Tuple<double, double>((t.X - p.X) / norm, (t.Y - p.Y) / norm));
                    }
                }
            }

            if (options.Count == 0)
                return p;

            int index = 0;
            int maxIndex = 0;
            double max = Double.NegativeInfinity;
            foreach (Tuple<double, double> t in options)
            {
                double dot = t.Item1 * direction.Item1 + t.Item2 * direction.Item2; //dot product
                if (dot > max)
                {
                    maxIndex = index;
                    max = dot;
                }
                index++;
            }

            //change the direction
            //direction = new Tuple<double, double>(options[maxIndex].Item1, options[maxIndex].Item2);            

            double newX = options[maxIndex].Item1 + Constant.lambda * direction.Item1;
            double newY = options[maxIndex].Item2 + Constant.lambda * direction.Item2;
            double newNorm = Math.Sqrt(newX * newX + newY * newY);
            newX = newX / newNorm;
            newY = newY / newNorm;
            direction = new Tuple<double, double>(newX, newY);
            //move along the new direction

            int u = (int)Math.Ceiling(Math.Abs(options[maxIndex].Item1));
            int v = (int)Math.Ceiling(Math.Abs(options[maxIndex].Item2));
            if (options[maxIndex].Item1 < 0)
                u = -1 * u;
            if (options[maxIndex].Item2 < 0)
                v = -1 * v;

            return moveKStep(new Point(p.X + u, p.Y + v),pos,lp);
        }

        static bool closed(Point p)
        {
            /* checks if this newly added point p is close to previous points.
             * if it is then we stop and trim the head of list till where it matches.*/
            int c = 0;
            foreach (Point t in newlyAddedPoint)
            {
                if (2 * c > step)
                    break;
                if ((p.X - t.X) * (p.X - t.X) + (p.Y - t.Y) * (p.Y - t.Y) < Constant.sqrMinDist)
                {
                    //remove all the points till here                    
                    newlyAddedPoint.RemoveRange(0, c);                    
                    return true;                    
                }
                c++;
            }
            return false;
        }


        static void move(Point p)
        {
            if (step>20 && closed(p))
            {
                newlyAddedPoint.Add(p);
                return;
            }
            newlyAddedPoint.Add(p);            
            if (step >= Constant.maxSteps)
                return;
            if (Constant.stepEnable && step % 30 == 0)
            {
                /*save interpretation*/
                img.Data[p.Y, p.X, 0] = 255;
                img.Data[p.Y, p.X, 1] = 255;
                img.Data[p.Y, p.X, 2] = 255;
                img.Save(@"C:\dataset\"+problem+@"\step\STEP"+colorIndex.ToString()+"_"+step.ToString()+".png");
                img.Data[p.Y, p.X, 0] = 0;
                img.Data[p.Y, p.X, 1] = 0;
                img.Data[p.Y, p.X, 2] = 255;
            }

            List<Point> neighbhors = giveNeighbhors(p);
            List<Point> future = new List<Point>();
            List<List<Point>> lp = new List<List<Point>>();
            int listindexer = 0;
            foreach (Point t in neighbhors)
            {
                int b = img.Data[t.Y, t.X, 0];
                int g = img.Data[t.Y, t.X, 1];
                int r = img.Data[t.Y, t.X, 2];
                if (b == 0 && g == 0 && r == 255)   //red pixel
                {
                    if (t.X != prev.X || t.Y != prev.Y) //not previous location
                    {
                        Tuple<double, double> copyDirection = new Tuple<double, double>(direction.Item1, direction.Item2);
                        List<Point> nlp = new List<Point>();
                        Point zzz = moveKStep(t,Constant.k,nlp);                        
                        future.Add(zzz);    //will change direction
                        listindexer++;
                        lp.Add(nlp);
                        direction = new Tuple<double, double>(copyDirection.Item1,copyDirection.Item2);
                    }
                }
            }

            int maxIndex = 0,index=0;            
            double maxDot = Double.NegativeInfinity;

            foreach (Point t in future)
            {
                /*find dot product with present direction*/
                double u = t.X-p.X;
                double v = t.Y-p.Y;
                double norm = Math.Sqrt(u*u+v*v);
                double dot = (direction.Item1 * u + direction.Item2 * v)/norm;
                if (maxDot < dot)
                {
                    maxDot = dot;
                    maxIndex = index;                    
                }
                index++;
            }

            foreach (Point u in lp[maxIndex])
            {
                newlyAddedPoint.Add(u);
            }

            double m = future[maxIndex].X - p.X;
            double n = future[maxIndex].Y - p.Y;
            double newNorm = Math.Sqrt(m*m+n*n);
            //Tuple<double, double> olddirection = direction;
            direction = new Tuple<double, double>(m/newNorm,n/newNorm);
            /*find continuity energy*/
            //double change=olddirection.Item1*direction.Item1+olddirection.Item2*direction.Item2;
            //nEnergy = nEnergy - change;            
            step++;
            move(new Point(future[maxIndex].X,future[maxIndex].Y));
        }

        static void saveNewlyAdded()
        {
            Image<Bgr, Byte> img = new Image<Bgr, Byte>(image.Width,image.Height);
            Image<Bgr, Byte> img1 = new Image<Bgr, Byte>(image.Width, image.Height);
            Color c=colorList[colorIndex%colorList.Count];
            foreach(Point t in newlyAddedPoint)
            {                
                List<Point> p=giveNeighbhors(t);
                p.Add(t);
                foreach (Point u in p)
                {
                    if (isMarked(u))
                    {
                        img.Data[u.Y, u.X, 0] = c.B;
                        img.Data[u.Y, u.X, 1] = c.G;
                        img.Data[u.Y, u.X, 2] = c.R;        
                        allPoint.Add(new Point(u.X, u.Y));
                    }                    
                }
                allPoint.Add(new Point(t.X,t.Y));            
            }

            /*if closed region then save everything inside the region*/
            MemStorage n = new MemStorage();
            Image<Gray, Byte> tmpimage = img.Convert<Gray,Byte>();
            Contour<Point> contour = new Contour<Point>(n);            
            contour = tmpimage.FindContours();            
            for (int y = 0; y < image.Height; y++)
            {
                for (int x = 0; x < image.Width; x++)
                {
                    if (contour.InContour(new Point(x, y))>=0)
                    {
                        img1.Data[y, x, 0] = image.Data[y, x, 0];
                        img1.Data[y, x, 1] = image.Data[y, x, 1];
                        img1.Data[y, x, 2] = image.Data[y, x, 2];
                    }
                }
            }            
            img1.Save(@"C:\dataset\Ans\"+problemNo.ToString()+@"\Continuum\" + colorIndex.ToString() + ".png");            
            //compute energies            
            continuumLawConstantEnergy.Add(constantComplexity(img1));
            continuumLawContinuumEnergy.Add(continuityComplexity(img1.Convert<Gray, Byte>()));
            newlyAddedPoint.Clear();
        }

        static bool isAway(Point p)
        {
            /*check if this point is away from all other points in the list*/
            double minDistance = Double.PositiveInfinity;            

            foreach (Point t in allPoint)
            {
                double dist=Math.Sqrt((p.X-t.X)*(p.X-t.X)+(p.Y-t.Y)*(p.Y-t.Y));
                if(dist < minDistance)
                {
                    minDistance=dist;
                }
            }

            if (minDistance > Constant.eta)
                return true;
            else return false;
        }

        static void startMoving()
        {            
            for (int x = 0; x < img.Width; x++)
            {
                for (int y = 0; y < img.Height; y++)
                {
                    int b = img.Data[y, x, 0];
                    int g = img.Data[y, x, 1];
                    int r = img.Data[y, x, 2];
                    if (b == 0 && g == 0 && r == 255)
                    {
                        if (isAway(new Point(x, y)))    //not from previous red points
                        {
                            step = 0;
                            prev = new Point(x, y);
                            nEnergy = 0; //nullify previous energy
                            direction = new Tuple<double, double>(1, 0); //begin with a random direction
                            move(new Point(x, y));                                                        
                            saveNewlyAdded();
                            maxObject2++;
                            if (maxObject2 > Constant.maxObjects)
                                return;
                            colorIndex++;
                            break;
                        }
                    }
                }
            }            
        }

        static Tuple<double,int,double> constantComplexity(Image<Bgr, Byte> tmpImage)
        {
            double complexity = 0;
            int foreground=0;
            for (int y = 0; y < tmpImage.Height; y++)
            {
                for (int x = 0; x < tmpImage.Width; x++)
                {
                    /*check for contrast*/
                    List<Point> ngbr = giveNeighbhors(new Point(x,y));
                    int b1 = tmpImage.Data[y, x, 0];
                    int g1 = tmpImage.Data[y, x, 1];
                    int r1 = tmpImage.Data[y, x, 2];
                    if (tmpImage.Data[y, x, 0] + tmpImage.Data[y, x, 1] + tmpImage.Data[y, x, 2] == 0)
                        continue;
                    foreground++;   //part of object
                    foreach (Point u in ngbr)
                    {
                        if (tmpImage.Data[u.Y, u.X, 0] + tmpImage.Data[u.Y, u.X, 1] + tmpImage.Data[u.Y, u.X, 2] == 0)
                        {
                            //ignores background
                            continue;
                        }
                        else
                        {
                            //check for contrast
                            int b2 = tmpImage.Data[u.Y, u.X, 0];
                            int g2 = tmpImage.Data[u.Y, u.X, 1];
                            int r2 = tmpImage.Data[u.Y, u.X, 2];
                            if (Math.Abs((double)(r1 - r2 + b1 - b2 + g1 - g2) / 3) > Constant.contrastThreshold)
                                complexity++;
                        }
                    }
                }
            }
            return new Tuple<double,int,double>(complexity,foreground,complexity/foreground);
        }

        static Tuple<double, int, double> continuityComplexity(Image<Gray, Byte> tmpImage)
        {
            /* assumes black color to be the background and there is only
             * one object in the image*/
            MemStorage n = new MemStorage();
            Contour<Point> t = new Contour<Point>(n);
            //CvInvoke.cvShowImage("tempimage", tmpImage);
            //CvInvoke.cvWaitKey(0);
            t = tmpImage.FindContours(Emgu.CV.CvEnum.CHAIN_APPROX_METHOD.CV_CHAIN_APPROX_NONE, Emgu.CV.CvEnum.RETR_TYPE.CV_RETR_LIST);
            if (t == null)
            {
                Console.WriteLine("\n\nNull");
                return new Tuple<double, int, double>(0, 0, 0);
            }
            List<Point> pt = t.ToList();
            Double contourEnergy = 0;
            for (int i = 0; i < pt.Count; i++)
            {
                /*find tangential difference*/
                int j2 = (i + 1)%pt.Count;
                int j3 = (i + 2) % pt.Count;
                Tuple<double,double> u=new Tuple<double,double>(pt[j2].X-pt[i].X,pt[j2].Y-pt[i].Y);
                Tuple<double, double> v = new Tuple<double, double>(pt[j3].X - pt[j2].X, pt[j3].Y - pt[j2].Y);
                /*normalization*/
                double normU=Math.Sqrt(u.Item1*u.Item1+u.Item2*u.Item2)+Constant.epsilon;                
                u = new Tuple<double, double>(u.Item1/normU,u.Item2/normU);

                double normV = Math.Sqrt(v.Item1 * v.Item1 + v.Item2 * v.Item2)+Constant.epsilon;                
                v = new Tuple<double, double>(v.Item1 / normV, v.Item2 / normV);

                /*difference*/
                contourEnergy += 1-(u.Item1 * v.Item1 + u.Item2 * v.Item2);  //value in between [0,2] with 0 meaning no energy gain
                                                                             //and 2 means high energy value
            }
            return new Tuple<double,int,double>(contourEnergy,pt.Count,contourEnergy/pt.Count);
        }


        static void Main(string[] args)
        {
            TextWriter tw = new StreamWriter(@"C:\dataset\results.txt");                        
            for (problemNo = 61; problemNo <=67; problemNo++)
            {
                index1 = 0;
                //create folders
                maxObject1 = 0;
                maxObject2 = 0;
                maxObject3 = 0;
                System.IO.Directory.CreateDirectory(@"C:\dataset\Ans\" + problemNo);
                System.IO.Directory.CreateDirectory(@"C:\dataset\Ans\" + problemNo + @"\ColorConstancy");
                System.IO.Directory.CreateDirectory(@"C:\dataset\Ans\" + problemNo + @"\Continuum");
                System.IO.Directory.CreateDirectory(@"C:\dataset\Ans\" + problemNo + @"\Proximity");

                image = new Image<Bgr, Byte>(@"C:\dataset\problems\image" + problemNo.ToString() + ".png");
                img = new Image<Bgr, Byte>(image.Width, image.Height);
                //CvInvoke.cvShowImage("tempimage", image);
                //CvInvoke.cvWaitKey(0);
                imgproxit = new Image<Bgr, Byte>(image.Width, image.Height);
                //Console.WriteLine("Values" + image.Data[0, 0, 0] + image.Data[0, 0, 1] + image.Data[0, 0, 2]);
                /* Gestalt Law of Color Constancy*/
                segment();
                GestaltConstantColor();     //call Gestalt Contast Color Law 
                img.Save(@"C:\dataset\Ans\" + problemNo + @"\boundaries.png");
                for (int x = 0; x < image.Width; x++)
                {
                    for (int y = 0; y < image.Height; y++)
                    {
                        imgproxit.Data[y, x, 0] = image.Data[y, x, 0];
                        imgproxit.Data[y, x, 1] = image.Data[y, x, 1];
                        imgproxit.Data[y, x, 2] = image.Data[y, x, 2];
                    }
                }
                GestaltProximity();
                /* Gestalt Law of Continuum*/
                startMoving();

                int whichObject = 0;

                Console.WriteLine("\n\nAnalysing Continuity Law");
                double continuumConstant = 0, continuumContinuum = 0, proximity=0;
                for (int i = 0; i < continuumLawContinuumEnergy.Count; i++)
                {
                    Tuple<double, int, double> t = continuumLawConstantEnergy.ElementAt(i);
                    Tuple<double, int, double> z = continuumLawContinuumEnergy.ElementAt(i);
                    Console.WriteLine("Object" + whichObject.ToString());
                    continuumConstant += t.Item3;
                    continuumContinuum += z.Item3;
                    Console.WriteLine("\tConstant Energy " + t.Item1 + " " + t.Item2 + "  " + t.Item3);
                    Console.WriteLine("\tContinuum Energy " + z.Item1 + " " + z.Item2 + "  " + z.Item3);
                    whichObject++;
                }
                continuumConstant /= continuumLawConstantEnergy.Count;
                continuumContinuum /= continuumLawContinuumEnergy.Count;

                whichObject = 0;
                Console.WriteLine("\n\nAnalysing Color Constancy Law");
                double constantConstant = 0, constantContinuum = 0;
                constantLawContinuumEnergy.RemoveAt(0);    //background
                constantLawConstantEnergy.RemoveAt(0);
                for (int i = 0; i < constantLawConstantEnergy.Count; i++)
                {
                    Tuple<double, int, double> t = constantLawConstantEnergy.ElementAt(i);
                    Tuple<double, int, double> z = constantLawContinuumEnergy.ElementAt(i);
                    Console.WriteLine("Object" + whichObject.ToString());
                    constantConstant += t.Item3;
                    constantContinuum += z.Item3;
                    Console.WriteLine("\tContrast Energy " + t.Item1 + " " + t.Item2 + "  " + t.Item3);
                    Console.WriteLine("\tContinuum Energy " + z.Item1 + " " + z.Item2 + "  " + z.Item3);
                    whichObject++;
                }
                constantConstant /= constantLawConstantEnergy.Count;
                constantContinuum /= constantLawContinuumEnergy.Count;
                if (problemNo >= 51 && problemNo <= 55)
                {
                    proximity = .765 / Math.Sqrt(problemNo - 49);
                }
                else if (problemNo >= 61 && problemNo <= 65)
                {
                    proximity = .45327 / Math.Sqrt(problemNo - 59);
                }
                else
                {
                    proximity = 0;
                }

                Console.WriteLine("Displaying Features");

                Console.WriteLine("Features for Constant Law");
                Console.WriteLine("\tConstant Features" + constantConstant);
                Console.WriteLine("\tContinuity Features" + constantContinuum);

                Console.WriteLine("Features for Continuity Law");
                Console.WriteLine("\tConstant Features" + continuumConstant);
                Console.WriteLine("\tContinuity Features" + continuumContinuum);

                tw.WriteLine(problemNo+","+constantConstant+","+constantContinuum+","+continuumConstant+","+continuumContinuum+","+proximity);
                tw.Flush();
                Console.WriteLine("Over");

                constantLawConstantEnergy.Clear();
                constantLawContinuumEnergy.Clear();
                continuumLawConstantEnergy.Clear();
                continuumLawContinuumEnergy.Clear();
                cluster.Clear();
                newlyAddedPoint.Clear();
                allPoint.Clear();
            }
            tw.Close();
        }
    }
}
